@@ -4,7 +4,7 @@
  * Copyright (C) 2003, 2004 Red Hat, Inc.
  *
  * Licensed under the Academic Free License version 2.1
- * 
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -14,13 +14,14 @@
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 
+#include <config.h>
 #include "bus.h"
 #include "activation.h"
 #include "connection.h"
@@ -33,7 +34,11 @@
 #include "dir-watch.h"
 #include <dbus/dbus-list.h>
 #include <dbus/dbus-hash.h>
+#include <dbus/dbus-credentials.h>
 #include <dbus/dbus-internals.h>
+#ifdef DBUS_CYGWIN
+#include <signal.h>
+#endif
 
 struct BusContext
 {
@@ -45,6 +50,7 @@ struct BusContext
   char *address;
   char *pidfile;
   char *user;
+  char *log_prefix;
   DBusLoop *loop;
   DBusList *servers;
   BusConnections *connections;
@@ -56,6 +62,8 @@ struct BusContext
   unsigned int fork : 1;
   unsigned int syslog : 1;
   unsigned int keep_umask : 1;
+  unsigned int allow_anonymous : 1;
+  unsigned int systemd_activation : 1;
 };
 
 static dbus_int32_t server_data_slot = -1;
@@ -72,7 +80,7 @@ server_get_context (DBusServer *server)
 {
   BusContext *context;
   BusServerData *bd;
-  
+
   if (!dbus_server_allocate_data_slot (&server_data_slot))
     return NULL;
 
@@ -99,7 +107,7 @@ server_watch_callback (DBusWatch     *watch,
    * if the code in activation.c for the babysitter
    * watch handler is fixed.
    */
-  
+
   return dbus_watch_handle (watch, condition);
 }
 
@@ -109,9 +117,9 @@ add_server_watch (DBusWatch  *watch,
 {
   DBusServer *server = data;
   BusContext *context;
-  
+
   context = server_get_context (server);
-  
+
   return _dbus_loop_add_watch (context->loop,
                                watch, server_watch_callback, server,
                                NULL);
@@ -123,9 +131,9 @@ remove_server_watch (DBusWatch  *watch,
 {
   DBusServer *server = data;
   BusContext *context;
-  
+
   context = server_get_context (server);
-  
+
   _dbus_loop_remove_watch (context->loop,
                            watch, server_watch_callback, server);
 }
@@ -145,7 +153,7 @@ add_server_timeout (DBusTimeout *timeout,
 {
   DBusServer *server = data;
   BusContext *context;
-  
+
   context = server_get_context (server);
 
   return _dbus_loop_add_timeout (context->loop,
@@ -158,9 +166,9 @@ remove_server_timeout (DBusTimeout *timeout,
 {
   DBusServer *server = data;
   BusContext *context;
-  
+
   context = server_get_context (server);
-  
+
   _dbus_loop_remove_timeout (context->loop,
                              timeout, server_timeout_callback, server);
 }
@@ -171,7 +179,7 @@ new_connection_callback (DBusServer     *server,
                          void           *data)
 {
   BusContext *context = data;
-  
+
   if (!bus_connections_setup_connection (context->connections, new_connection))
     {
       _dbus_verbose ("No memory to setup new connection\n");
@@ -189,15 +197,24 @@ new_connection_callback (DBusServer     *server,
 
   dbus_connection_set_max_message_size (new_connection,
                                         context->limits.max_message_size);
-  
+
+  dbus_connection_set_max_received_unix_fds (new_connection,
+                                         context->limits.max_incoming_unix_fds);
+
+  dbus_connection_set_max_message_unix_fds (new_connection,
+                                        context->limits.max_message_unix_fds);
+
+  dbus_connection_set_allow_anonymous (new_connection,
+                                       context->allow_anonymous);
+
   /* on OOM, we won't have ref'd the connection so it will die. */
 }
 
 static void
 free_server_data (void *data)
 {
-  BusServerData *bd = data;  
-  
+  BusServerData *bd = data;
+
   dbus_free (bd);
 }
 
@@ -220,17 +237,17 @@ setup_server (BusContext *context,
     }
 
   bd->context = context;
-  
+
   if (!dbus_server_set_auth_mechanisms (server, (const char**) auth_mechanisms))
     {
       BUS_SET_OOM (error);
       return FALSE;
     }
-  
+
   dbus_server_set_new_connection_function (server,
                                            new_connection_callback,
                                            context, NULL);
-  
+
   if (!dbus_server_set_watch_functions (server,
                                         add_server_watch,
                                         remove_server_watch,
@@ -251,7 +268,7 @@ setup_server (BusContext *context,
       BUS_SET_OOM (error);
       return FALSE;
     }
-  
+
   return TRUE;
 }
 
@@ -260,10 +277,13 @@ setup_server (BusContext *context,
  * when config files are reloaded.
  */
 static dbus_bool_t
-process_config_first_time_only (BusContext      *context,
-				BusConfigParser *parser,
-				DBusError       *error)
+process_config_first_time_only (BusContext       *context,
+				BusConfigParser  *parser,
+                                const DBusString *address,
+                                dbus_bool_t      systemd_activation,
+				DBusError        *error)
 {
+  DBusString log_prefix;
   DBusList *link;
   DBusList **addresses;
   const char *user, *pidfile;
@@ -277,6 +297,8 @@ process_config_first_time_only (BusContext      *context,
   retval = FALSE;
   auth_mechanisms = NULL;
 
+  context->systemd_activation = systemd_activation;
+
   /* Check for an existing pid file. Of course this is a race;
    * we'd have to use fcntl() locks on the pid file to
    * avoid that. But we want to check for the pid file
@@ -287,23 +309,82 @@ process_config_first_time_only (BusContext      *context,
     {
       DBusString u;
       DBusStat stbuf;
-      
+
       _dbus_string_init_const (&u, pidfile);
-      
+
       if (_dbus_stat (&u, &stbuf, NULL))
-	{
-	  dbus_set_error (error, DBUS_ERROR_FAILED,
-			  "The pid file \"%s\" exists, if the message bus is not running, remove this file",
-			  pidfile);
-	  goto failed;
-	}
+        {
+#ifdef DBUS_CYGWIN
+          DBusString p;
+          long /* int */ pid;
+
+          _dbus_string_init (&p);
+          _dbus_file_get_contents(&p, &u, NULL);
+          _dbus_string_parse_int(&p, 0, &pid, NULL);
+          _dbus_string_free(&p);
+
+          if ((kill((int)pid, 0))) {
+            dbus_set_error(NULL, DBUS_ERROR_FILE_EXISTS,
+                           "pid %ld not running, removing stale pid file\n",
+                           pid);
+            _dbus_delete_file(&u, NULL);
+          } else {
+#endif
+          dbus_set_error (error, DBUS_ERROR_FAILED,
+		                  "The pid file \"%s\" exists, if the message bus is not running, remove this file",
+                          pidfile);
+	      goto failed;
+#ifdef DBUS_CYGWIN
+          }
+#endif
+        }
     }
-  
+
   /* keep around the pid filename so we can delete it later */
   context->pidfile = _dbus_strdup (pidfile);
 
+  /* note that type may be NULL */
+  context->type = _dbus_strdup (bus_config_parser_get_type (parser));
+  if (bus_config_parser_get_type (parser) != NULL && context->type == NULL)
+    goto oom;
+
+  user = bus_config_parser_get_user (parser);
+  if (user != NULL)
+    {
+      context->user = _dbus_strdup (user);
+      if (context->user == NULL)
+        goto oom;
+    }
+
+  /* Set up the prefix for syslog messages */
+  if (!_dbus_string_init (&log_prefix))
+    goto oom;
+  if (context->type && !strcmp (context->type, "system"))
+    {
+      if (!_dbus_string_append (&log_prefix, "[system] "))
+        goto oom;
+    }
+  else if (context->type && !strcmp (context->type, "session"))
+    {
+      DBusCredentials *credentials;
+
+      credentials = _dbus_credentials_new_from_current_process ();
+      if (!credentials)
+        goto oom;
+      if (!_dbus_string_append (&log_prefix, "[session "))
+        goto oom;
+      if (!_dbus_credentials_to_string_append (credentials, &log_prefix))
+        goto oom;
+      if (!_dbus_string_append (&log_prefix, "] "))
+        goto oom;
+      _dbus_credentials_unref (credentials);
+    }
+  if (!_dbus_string_steal_data (&log_prefix, &context->log_prefix))
+    goto oom;
+  _dbus_string_free (&log_prefix);
+
   /* Build an array of auth mechanisms */
-  
+
   auth_mechanisms_list = bus_config_parser_get_mechanisms (parser);
   len = _dbus_list_get_length (auth_mechanisms_list);
 
@@ -313,21 +394,15 @@ process_config_first_time_only (BusContext      *context,
 
       auth_mechanisms = dbus_new0 (char*, len + 1);
       if (auth_mechanisms == NULL)
-	{
-	  BUS_SET_OOM (error);
-	  goto failed;
-	}
-      
+        goto oom;
+
       i = 0;
       link = _dbus_list_get_first_link (auth_mechanisms_list);
       while (link != NULL)
         {
           auth_mechanisms[i] = _dbus_strdup (link->data);
           if (auth_mechanisms[i] == NULL)
-	    {
-	      BUS_SET_OOM (error);
-	      goto failed;
-	    }
+            goto oom;
           link = _dbus_list_get_next_link (auth_mechanisms_list, link);
         }
     }
@@ -337,64 +412,70 @@ process_config_first_time_only (BusContext      *context,
     }
 
   /* Listen on our addresses */
-  
-  addresses = bus_config_parser_get_addresses (parser);  
-  
-  link = _dbus_list_get_first_link (addresses);
-  while (link != NULL)
+
+  if (address)
     {
       DBusServer *server;
-      
-      server = dbus_server_listen (link->data, error);
+
+      server = dbus_server_listen (_dbus_string_get_const_data(address), error);
       if (server == NULL)
-	{
-	  _DBUS_ASSERT_ERROR_IS_SET (error);
-	  goto failed;
-	}
+        {
+          _DBUS_ASSERT_ERROR_IS_SET (error);
+          goto failed;
+        }
       else if (!setup_server (context, server, auth_mechanisms, error))
-	{
-	  _DBUS_ASSERT_ERROR_IS_SET (error);
-	  goto failed;
-	}
+        {
+          _DBUS_ASSERT_ERROR_IS_SET (error);
+          goto failed;
+        }
 
       if (!_dbus_list_append (&context->servers, server))
+        goto oom;
+    }
+  else
+    {
+      addresses = bus_config_parser_get_addresses (parser);
+
+      link = _dbus_list_get_first_link (addresses);
+      while (link != NULL)
         {
-          BUS_SET_OOM (error);
-          goto failed;
-        }          
-      
-      link = _dbus_list_get_next_link (addresses, link);
-    }
+          DBusServer *server;
 
-  /* note that type may be NULL */
-  context->type = _dbus_strdup (bus_config_parser_get_type (parser));
-  if (bus_config_parser_get_type (parser) != NULL && context->type == NULL)
-    {
-      BUS_SET_OOM (error);
-      goto failed;
-    }
+          server = dbus_server_listen (link->data, error);
+          if (server == NULL)
+            {
+              _DBUS_ASSERT_ERROR_IS_SET (error);
+              goto failed;
+            }
+          else if (!setup_server (context, server, auth_mechanisms, error))
+            {
+              _DBUS_ASSERT_ERROR_IS_SET (error);
+              goto failed;
+            }
 
-  user = bus_config_parser_get_user (parser);
-  if (user != NULL)
-    {
-      context->user = _dbus_strdup (user);
-      if (context->user == NULL)
-	{
-	  BUS_SET_OOM (error);
-	  goto failed;
-	}
+          if (!_dbus_list_append (&context->servers, server))
+            goto oom;
+
+          link = _dbus_list_get_next_link (addresses, link);
+        }
     }
 
   context->fork = bus_config_parser_get_fork (parser);
   context->syslog = bus_config_parser_get_syslog (parser);
   context->keep_umask = bus_config_parser_get_keep_umask (parser);
-  
+  context->allow_anonymous = bus_config_parser_get_allow_anonymous (parser);
+
   _DBUS_ASSERT_ERROR_IS_CLEAR (error);
   retval = TRUE;
 
  failed:
   dbus_free_string_array (auth_mechanisms);
   return retval;
+
+ oom:
+  BUS_SET_OOM (error);
+  dbus_free_string_array (auth_mechanisms);
+  return FALSE;
 }
 
 /* This code gets executed every time the config files
@@ -416,7 +497,7 @@ process_config_every_time (BusContext      *context,
   char *addr;
   const char *servicehelper;
   char *s;
-  
+
   dbus_bool_t retval;
 
   _DBUS_ASSERT_ERROR_IS_CLEAR (error);
@@ -433,6 +514,8 @@ process_config_every_time (BusContext      *context,
   /* get our limits and timeout lengths */
   bus_config_parser_get_limits (parser, &context->limits);
 
+  if (context->policy)
+    bus_policy_unref (context->policy);
   context->policy = bus_config_parser_steal_policy (parser);
   _dbus_assert (context->policy != NULL);
 
@@ -496,44 +579,70 @@ process_config_every_time (BusContext      *context,
       dbus_free(context->servicehelper);
       context->servicehelper = s;
     }
-  
+
   /* Create activation subsystem */
-  new_activation = bus_activation_new (context, &full_address,
-                                       dirs, error);
-  if (new_activation == NULL)
+  if (context->activation)
+    {
+      if (!bus_activation_reload (context->activation, &full_address, dirs, error))
+        goto failed;
+    }
+  else
+    {
+      context->activation = bus_activation_new (context, &full_address, dirs, error);
+    }
+
+  if (context->activation == NULL)
     {
       _DBUS_ASSERT_ERROR_IS_SET (error);
       goto failed;
     }
 
-  if (is_reload)
-    bus_activation_unref (context->activation);
-
-  context->activation = new_activation;
-
-  /* Drop existing conf-dir watches (if applicable) */
-
-  if (is_reload)
-    bus_drop_all_directory_watches ();
-
   _DBUS_ASSERT_ERROR_IS_CLEAR (error);
   retval = TRUE;
 
  failed:
   _dbus_string_free (&full_address);
-  
+
   if (addr)
     dbus_free (addr);
 
   return retval;
 }
 
+static dbus_bool_t
+list_concat_new (DBusList **a,
+                 DBusList **b,
+                 DBusList **result)
+{
+  DBusList *link;
+
+  *result = NULL;
+
+  link = _dbus_list_get_first_link (a);
+  for (link = _dbus_list_get_first_link (a); link; link = _dbus_list_get_next_link (a, link))
+    {
+      if (!_dbus_list_append (result, link->data))
+        goto oom;
+    }
+  for (link = _dbus_list_get_first_link (b); link; link = _dbus_list_get_next_link (b, link))
+    {
+      if (!_dbus_list_append (result, link->data))
+        goto oom;
+    }
+
+  return TRUE;
+oom:
+  _dbus_list_clear (result);
+  return FALSE;
+}
+
 static dbus_bool_t
 process_config_postinit (BusContext      *context,
 			 BusConfigParser *parser,
 			 DBusError       *error)
 {
   DBusHashTable *service_context_table;
+  DBusList *watched_dirs = NULL;
 
   service_context_table = bus_config_parser_steal_service_context_table (parser);
   if (!bus_registry_set_service_context_table (context->registry,
@@ -545,10 +654,20 @@ process_config_postinit (BusContext      *context,
 
   _dbus_hash_table_unref (service_context_table);
 
-  /* Watch all conf directories */
-  _dbus_list_foreach (bus_config_parser_get_conf_dirs (parser),
-		      (DBusForeachFunction) bus_watch_directory,
-		      context);
+  /* We need to monitor both the configuration directories and directories
+   * containing .service files.
+   */
+  if (!list_concat_new (bus_config_parser_get_conf_dirs (parser),
+                        bus_config_parser_get_service_dirs (parser),
+                        &watched_dirs))
+    {
+      BUS_SET_OOM (error);
+      return FALSE;
+    }
+
+  bus_set_watched_dirs (context, &watched_dirs);
+
+  _dbus_list_clear (&watched_dirs);
 
   return TRUE;
 }
@@ -558,8 +677,11 @@ bus_context_new (const DBusString *config_file,
                  ForceForkSetting  force_fork,
                  DBusPipe         *print_addr_pipe,
                  DBusPipe         *print_pid_pipe,
+                 const DBusString *address,
+                 dbus_bool_t      systemd_activation,
                  DBusError        *error)
 {
+  DBusString log_prefix;
   BusContext *context;
   BusConfigParser *parser;
 
@@ -583,7 +705,7 @@ bus_context_new (const DBusString *config_file,
   context->refcount = 1;
 
   _dbus_generate_uuid (&context->uuid);
-  
+
   if (!_dbus_string_copy_data (config_file, &context->config_file))
     {
       BUS_SET_OOM (error);
@@ -610,8 +732,8 @@ bus_context_new (const DBusString *config_file,
       _DBUS_ASSERT_ERROR_IS_SET (error);
       goto failed;
     }
-  
-  if (!process_config_first_time_only (context, parser, error))
+
+  if (!process_config_first_time_only (context, parser, address, systemd_activation, error))
     {
       _DBUS_ASSERT_ERROR_IS_SET (error);
       goto failed;
@@ -621,7 +743,7 @@ bus_context_new (const DBusString *config_file,
       _DBUS_ASSERT_ERROR_IS_SET (error);
       goto failed;
     }
-  
+
   /* we need another ref of the server data slot for the context
    * to own
    */
@@ -638,14 +760,14 @@ bus_context_new (const DBusString *config_file,
       DBusString addr;
       const char *a = bus_context_get_address (context);
       int bytes;
-      
+
       _dbus_assert (a != NULL);
       if (!_dbus_string_init (&addr))
         {
           BUS_SET_OOM (error);
           goto failed;
         }
-      
+
       if (!_dbus_string_append (&addr, a) ||
           !_dbus_string_append (&addr, "\n"))
         {
@@ -669,10 +791,10 @@ bus_context_new (const DBusString *config_file,
 
       if (!_dbus_pipe_is_stdout_or_stderr (print_addr_pipe))
         _dbus_pipe_close (print_addr_pipe, NULL);
-      
+
       _dbus_string_free (&addr);
     }
-  
+
   context->connections = bus_connections_new (context);
   if (context->connections == NULL)
     {
@@ -709,8 +831,8 @@ bus_context_new (const DBusString *config_file,
     if ((force_fork != FORK_NEVER && context->fork) || force_fork == FORK_ALWAYS)
       {
         _dbus_verbose ("Forking and becoming daemon\n");
-        
-        if (!_dbus_become_daemon (context->pidfile ? &u : NULL, 
+
+        if (!_dbus_become_daemon (context->pidfile ? &u : NULL,
                                   print_pid_pipe,
                                   error,
                                   context->keep_umask))
@@ -722,7 +844,7 @@ bus_context_new (const DBusString *config_file,
     else
       {
         _dbus_verbose ("Fork not requested\n");
-        
+
         /* Need to write PID file and to PID pipe for ourselves,
          * not for the child process. This is a no-op if the pidfile
          * is NULL and print_pid_pipe is NULL.
@@ -744,9 +866,9 @@ bus_context_new (const DBusString *config_file,
 
   if (!bus_selinux_full_init ())
     {
-      _dbus_warn ("SELinux initialization failed\n");
+      bus_context_log (context, DBUS_SYSTEM_LOG_FATAL, "SELinux enabled but AVC initialization failed; check system log\n");
     }
-  
+
   if (!process_config_postinit (context, parser, error))
     {
       _DBUS_ASSERT_ERROR_IS_SET (error);
@@ -758,7 +880,7 @@ bus_context_new (const DBusString *config_file,
       bus_config_parser_unref (parser);
       parser = NULL;
     }
-  
+
   /* Here we change our credentials if required,
    * as soon as we've set up our sockets and pidfile
    */
@@ -777,10 +899,10 @@ bus_context_new (const DBusString *config_file,
     }
 
   dbus_server_free_data_slot (&server_data_slot);
-  
+
   return context;
-  
- failed:  
+
+ failed:
   if (parser != NULL)
     bus_config_parser_unref (parser);
   if (context != NULL)
@@ -788,7 +910,7 @@ bus_context_new (const DBusString *config_file,
 
   if (server_data_slot >= 0)
     dbus_server_free_data_slot (&server_data_slot);
-  
+
   return NULL;
 }
 
@@ -818,7 +940,7 @@ bus_context_reload_config (BusContext *context,
       _DBUS_ASSERT_ERROR_IS_SET (error);
       goto failed;
     }
-  
+
   if (!process_config_every_time (context, parser, TRUE, error))
     {
       _DBUS_ASSERT_ERROR_IS_SET (error);
@@ -831,10 +953,10 @@ bus_context_reload_config (BusContext *context,
     }
   ret = TRUE;
 
-  bus_context_log_info (context, "Reloaded configuration");
- failed:  
+  bus_context_log (context, DBUS_SYSTEM_LOG_INFO, "Reloaded configuration");
+ failed:
   if (!ret)
-    bus_context_log_info (context, "Unable to reload configuration: %s", error->message);
+    bus_context_log (context, DBUS_SYSTEM_LOG_INFO, "Unable to reload configuration: %s", error->message);
   if (parser != NULL)
     bus_config_parser_unref (parser);
   return ret;
@@ -847,19 +969,19 @@ shutdown_server (BusContext *context,
   if (server == NULL ||
       !dbus_server_get_is_connected (server))
     return;
-  
+
   if (!dbus_server_set_watch_functions (server,
                                         NULL, NULL, NULL,
                                         context,
                                         NULL))
     _dbus_assert_not_reached ("setting watch functions to NULL failed");
-  
+
   if (!dbus_server_set_timeout_functions (server,
                                           NULL, NULL, NULL,
                                           context,
                                           NULL))
     _dbus_assert_not_reached ("setting timeout functions to NULL failed");
-  
+
   dbus_server_disconnect (server);
 }
 
@@ -895,9 +1017,9 @@ bus_context_unref (BusContext *context)
   if (context->refcount == 0)
     {
       DBusList *link;
-      
+
       _dbus_verbose ("Finalizing bus context %p\n", context);
-      
+
       bus_context_shutdown (context);
 
       if (context->connections)
@@ -905,13 +1027,13 @@ bus_context_unref (BusContext *context)
           bus_connections_unref (context->connections);
           context->connections = NULL;
         }
-      
+
       if (context->registry)
         {
           bus_registry_unref (context->registry);
           context->registry = NULL;
         }
-      
+
       if (context->activation)
         {
           bus_activation_unref (context->activation);
@@ -922,7 +1044,7 @@ bus_context_unref (BusContext *context)
       while (link != NULL)
         {
           dbus_server_unref (link->data);
-          
+
           link = _dbus_list_get_next_link (&context->servers, link);
         }
       _dbus_list_clear (&context->servers);
@@ -932,7 +1054,7 @@ bus_context_unref (BusContext *context)
           bus_policy_unref (context->policy);
           context->policy = NULL;
         }
-      
+
       if (context->loop)
         {
           _dbus_loop_unref (context->loop);
@@ -944,8 +1066,9 @@ bus_context_unref (BusContext *context)
           bus_matchmaker_unref (context->matchmaker);
           context->matchmaker = NULL;
         }
-      
+
       dbus_free (context->config_file);
+      dbus_free (context->log_prefix);
       dbus_free (context->type);
       dbus_free (context->address);
       dbus_free (context->user);
@@ -961,7 +1084,7 @@ bus_context_unref (BusContext *context)
 	   */
 	  _dbus_delete_file (&u, NULL);
 
-          dbus_free (context->pidfile); 
+          dbus_free (context->pidfile);
 	}
       dbus_free (context);
 
@@ -988,6 +1111,12 @@ bus_context_get_servicehelper (BusContext *context)
   return context->servicehelper;
 }
 
+dbus_bool_t
+bus_context_get_systemd_activation (BusContext *context)
+{
+  return context->systemd_activation;
+}
+
 BusRegistry*
 bus_context_get_registry (BusContext  *context)
 {
@@ -1057,7 +1186,7 @@ bus_context_create_client_policy (BusContext      *context,
 int
 bus_context_get_activation_timeout (BusContext *context)
 {
-  
+
   return context->limits.activation_timeout;
 }
 
@@ -1116,28 +1245,37 @@ bus_context_get_reply_timeout (BusContext *context)
 }
 
 void
-bus_context_log_info (BusContext *context, const char *msg, ...)
-{
-  va_list args;
-
-  va_start (args, msg);
-  
-  if (context->syslog)
-    _dbus_log_info (msg, args);
-
-  va_end (args);
-}
+bus_context_log (BusContext *context, DBusSystemLogSeverity severity, const char *msg, ...) _DBUS_GNUC_PRINTF (3, 4);
 
 void
-bus_context_log_security (BusContext *context, const char *msg, ...)
+bus_context_log (BusContext *context, DBusSystemLogSeverity severity, const char *msg, ...)
 {
   va_list args;
 
-  va_start (args, msg);
-  
-  if (context->syslog)
-    _dbus_log_security (msg, args);
+  if (!context->syslog)
+    return;
 
+  va_start (args, msg);
+
+  if (context->log_prefix)
+    {
+      DBusString full_msg;
+
+      if (!_dbus_string_init (&full_msg))
+        goto out;
+      if (!_dbus_string_append (&full_msg, context->log_prefix))
+        goto oom_out;
+      if (!_dbus_string_append_printf_valist (&full_msg, msg, args))
+        goto oom_out;
+
+      _dbus_system_log (severity, "%s", _dbus_string_get_const_data (&full_msg));
+    oom_out:
+      _dbus_string_free (&full_msg);
+    }
+  else
+    _dbus_system_logv (severity, msg, args);
+
+out:
   va_end (args);
 }
 
@@ -1173,10 +1311,10 @@ bus_context_check_security_policy (BusContext     *context,
   const char *sender_name;
   const char *sender_loginfo;
   const char *proposed_recipient_loginfo;
-  
+
   type = dbus_message_get_type (message);
   dest = dbus_message_get_destination (message);
-  
+
   /* dispatch.c was supposed to ensure these invariants */
   _dbus_assert (dest != NULL ||
                 type == DBUS_MESSAGE_TYPE_SIGNAL ||
@@ -1196,12 +1334,12 @@ bus_context_check_security_policy (BusContext     *context,
       sender_name = NULL;
       sender_loginfo = "(bus)";
     }
-  
+
   if (proposed_recipient != NULL)
     proposed_recipient_loginfo = bus_connection_get_loginfo (proposed_recipient);
   else
     proposed_recipient_loginfo = "bus";
-  
+
   switch (type)
     {
     case DBUS_MESSAGE_TYPE_METHOD_CALL:
@@ -1209,19 +1347,19 @@ bus_context_check_security_policy (BusContext     *context,
     case DBUS_MESSAGE_TYPE_METHOD_RETURN:
     case DBUS_MESSAGE_TYPE_ERROR:
       break;
-      
+
     default:
       _dbus_verbose ("security check disallowing message of unknown type %d\n",
                      type);
 
       dbus_set_error (error, DBUS_ERROR_ACCESS_DENIED,
                       "Message bus will not accept messages of unknown type\n");
-              
+
       return FALSE;
     }
 
   requested_reply = FALSE;
-  
+
   if (sender != NULL)
     {
       /* First verify the SELinux access controls.  If allowed then
@@ -1254,12 +1392,12 @@ bus_context_check_security_policy (BusContext     *context,
 
           return FALSE;
         }
-       
+
       if (bus_connection_is_active (sender))
         {
           sender_policy = bus_connection_get_policy (sender);
           _dbus_assert (sender_policy != NULL);
-          
+
           /* Fill in requested_reply variable with TRUE if this is a
            * reply and the reply was pending.
            */
@@ -1268,8 +1406,8 @@ bus_context_check_security_policy (BusContext     *context,
               if (proposed_recipient != NULL /* not to the bus driver */ &&
                   addressed_recipient == proposed_recipient /* not eavesdropping */)
                 {
-                  DBusError error2;                  
-                  
+                  DBusError error2;
+
                   dbus_error_init (&error2);
                   requested_reply = bus_connections_check_reply (bus_connection_get_connections (sender),
                                                                  transaction,
@@ -1305,7 +1443,7 @@ bus_context_check_security_policy (BusContext     *context,
               dbus_set_error (error, DBUS_ERROR_ACCESS_DENIED,
                               "Client tried to send a message other than %s without being registered",
                               "Hello");
-              
+
               return FALSE;
             }
         }
@@ -1324,7 +1462,7 @@ bus_context_check_security_policy (BusContext     *context,
 
   _dbus_assert ((sender != NULL && sender_policy != NULL) ||
                 (sender == NULL && sender_policy == NULL));
-  
+
   if (proposed_recipient != NULL)
     {
       /* only the bus driver can send to an inactive recipient (as it
@@ -1349,11 +1487,11 @@ bus_context_check_security_policy (BusContext     *context,
     }
   else
     recipient_policy = NULL;
-  
+
   _dbus_assert ((proposed_recipient != NULL && recipient_policy != NULL) ||
                 (proposed_recipient != NULL && sender == NULL && recipient_policy == NULL) ||
                 (proposed_recipient == NULL && recipient_policy == NULL));
-  
+
   log = FALSE;
   if (sender_policy &&
       !bus_client_policy_check_can_send (sender_policy,
@@ -1380,8 +1518,8 @@ bus_context_check_security_policy (BusContext     *context,
                       dest ? dest : DBUS_SERVICE_DBUS,
                       proposed_recipient_loginfo);
       /* Needs to be duplicated to avoid calling malloc and having to handle OOM */
-      if (addressed_recipient == proposed_recipient)      
-        bus_context_log_security (context, msg,
+      if (addressed_recipient == proposed_recipient)
+        bus_context_log (context, DBUS_SYSTEM_LOG_SECURITY, msg,
                                   toggles,
                                   dbus_message_type_to_string (dbus_message_get_type (message)),
                                   sender_name ? sender_name : "(unset)",
@@ -1400,7 +1538,7 @@ bus_context_check_security_policy (BusContext     *context,
     }
 
   if (log)
-    bus_context_log_security (context, 
+    bus_context_log (context, DBUS_SYSTEM_LOG_SECURITY,
                               "Would reject message, %d matched rules; "
                               "type=\"%s\", sender=\"%s\" (%s) interface=\"%s\" member=\"%s\" error name=\"%s\" requested_reply=%d destination=\"%s\" (%s))",
                               toggles,
@@ -1413,7 +1551,7 @@ bus_context_check_security_policy (BusContext     *context,
                               dbus_message_get_member (message) : "(unset)",
                               dbus_message_get_error_name (message) ?
                               dbus_message_get_error_name (message) : "(unset)",
-                              requested_reply,                               
+                              requested_reply,
                               dest ? dest : DBUS_SERVICE_DBUS,
                               proposed_recipient_loginfo);
 
@@ -1444,8 +1582,8 @@ bus_context_check_security_policy (BusContext     *context,
                       dest ? dest : DBUS_SERVICE_DBUS,
                       proposed_recipient_loginfo);
       /* Needs to be duplicated to avoid calling malloc and having to handle OOM */
-      if (addressed_recipient == proposed_recipient)      
-        bus_context_log_security (context, msg,
+      if (addressed_recipient == proposed_recipient)
+        bus_context_log (context, DBUS_SYSTEM_LOG_SECURITY, msg,
                                   toggles,
                                   dbus_message_type_to_string (dbus_message_get_type (message)),
                                   sender_name ? sender_name : "(unset)",
@@ -1466,13 +1604,13 @@ bus_context_check_security_policy (BusContext     *context,
 
   /* See if limits on size have been exceeded */
   if (proposed_recipient &&
-      dbus_connection_get_outgoing_size (proposed_recipient) >
-      context->limits.max_outgoing_bytes)
+      ((dbus_connection_get_outgoing_size (proposed_recipient) > context->limits.max_outgoing_bytes) ||
+       (dbus_connection_get_outgoing_unix_fds (proposed_recipient) > context->limits.max_outgoing_unix_fds)))
     {
       dbus_set_error (error, DBUS_ERROR_LIMITS_EXCEEDED,
                       "The destination service \"%s\" has a full message queue",
                       dest ? dest : (proposed_recipient ?
-                                     bus_connection_get_name (proposed_recipient) : 
+                                     bus_connection_get_name (proposed_recipient) :
                                      DBUS_SERVICE_DBUS));
       _dbus_verbose ("security policy disallowing message due to full message queue\n");
       return FALSE;
@@ -1483,7 +1621,7 @@ bus_context_check_security_policy (BusContext     *context,
    * connection). Only the addressed recipient may reply.
    */
   if (type == DBUS_MESSAGE_TYPE_METHOD_CALL &&
-      sender && 
+      sender &&
       addressed_recipient &&
       addressed_recipient == proposed_recipient && /* not eavesdropping */
       !bus_connections_expect_reply (bus_connection_get_connections (sender),
@@ -1494,7 +1632,7 @@ bus_context_check_security_policy (BusContext     *context,
       _dbus_verbose ("Failed to record reply expectation or problem with the message expecting a reply\n");
       return FALSE;
     }
-  
+
   _dbus_verbose ("security policy allowing message\n");
   return TRUE;
 }
